/**
 * Copyright (c) 2006-2012 EBM WebSourcing, 2012-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.bc.mail.listeners;


import java.util.logging.Level;

import javax.jbi.JBIException;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessagingException;
import javax.xml.namespace.QName;

import org.ow2.petals.bc.mail.MailComponent;
import org.ow2.petals.bc.mail.MailConstants;
import org.ow2.petals.bc.mail.MailSessionManager;
import org.ow2.petals.bc.mail.MimeMessageManager;
import org.ow2.petals.bc.mail.service.provide.AbstractSendService;
import org.ow2.petals.bc.mail.service.provide.NativeSendService;
import org.ow2.petals.bc.mail.service.provide.SuSendService;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.logger.Utils;
import org.ow2.petals.component.framework.util.WSDLUtilImpl;

import com.ebmwebsourcing.easycommons.lang.StringHelper;

/**
 * Mail BC listener for incoming JBI messages
 * 
 * @author Olivier Fabre - EBM WebSourcing
 * @author Mathieu CARROLLE - EBM WebSourcing
 */
public class JBIListener extends AbstractJBIListener {

    protected MailSessionManager mailSessionManager;

    protected MimeMessageManager mimeMessageManager;

    protected NativeSendService sendMailService;

    /**
     * Validate the operation.
     * 
     * @param operation
     */
    private static final void validateExchange(final Exchange exchange) throws MessagingException {
        final QName operation = exchange.getOperation();
        // Null operation
        if (operation == null) {
            throw new MessagingException(
                    "The JBI operation must be set by the consumer of the service.");
        }
        // if no namespace is provided we use the default namespace.
        if (StringHelper.isNullOrEmpty(operation.getNamespaceURI())) {
            exchange.setOperation(new QName(MailConstants.MAIL_SERVICE_NS, operation.getLocalPart()));
        }
    }

    @Override
    public void init() {
        this.mimeMessageManager = ((MailComponent) this.getComponent()).getMimeMessageManager();
        this.mailSessionManager = ((MailComponent) this.getComponent()).getMailSessionManager();
        this.sendMailService = new NativeSendService(this.mailSessionManager,
                this.mimeMessageManager, this.getLogger());
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.objectweb.petals.component.common.bc.JBIListener#onJBIMessage(java
     * .lang.String,
     * org.objectweb.petals.component.common.util.MessageExchangeWrapper,
     * org.objectweb.petals.tools.jbicommon.descriptor.Extensions)
     */
    @Override
    public boolean onJBIMessage(final Exchange exchange) {
        if (exchange.isActiveStatus()) {

            AbstractSendService service = null;
            // Fault ignored
            if (exchange.getFault() == null) {
                try {
                    if (!exchange.isInOnlyPattern()) {
                        throw new JBIException("Exchange pattern for the "
                                + exchange.getOperation() + " operation must be "
                                + MEPConstants.IN_ONLY_PATTERN);
                    }
                    validateExchange(exchange);
                    if ((this.component.getNativeWsdl() != null) 
                            && (WSDLUtilImpl.isDescriptionContaining(this.component.getNativeWsdl()
                            .getDescription(), exchange.getEndpoint()))) {
                        service = new NativeSendService(mailSessionManager, mimeMessageManager,
                                getLogger());
                    } else {
                        service = new SuSendService(mailSessionManager, mimeMessageManager,
                                getLogger(), getExtensions());
                    }
                    service.sendMail(exchange);
                } catch (final Exception e) {
                    this.getLogger().log(Level.WARNING,
                            e.getClass().getSimpleName() + " : " + e.getMessage());
                    exchange.setError(e);
                }
            }
        } else if(!ExchangeStatus.ACTIVE.equals(exchange.getStatus()) && exchange.isConsumerRole()){
            Utils.addMonitEndOrFailureTrace(getLogger(), exchange, exchange.getFlowAttributes());
        }
        
        return true;
    }
}
